"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Device(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    ICON_FIELD_NUMBER: builtins.int
    INSERTTIME_FIELD_NUMBER: builtins.int
    FAVOURITE_FIELD_NUMBER: builtins.int
    ASSIGNED_PLACE_IDS_FIELD_NUMBER: builtins.int
    ASSIGNED_GROUP_IDS_FIELD_NUMBER: builtins.int
    id: builtins.int
    name: builtins.str
    type: builtins.int
    icon: builtins.int
    inserttime: builtins.int
    favourite: builtins.bool
    @property
    def assigned_place_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def assigned_group_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        id: builtins.int | None = ...,
        name: builtins.str | None = ...,
        type: builtins.int | None = ...,
        icon: builtins.int | None = ...,
        inserttime: builtins.int | None = ...,
        favourite: builtins.bool | None = ...,
        assigned_place_ids: collections.abc.Iterable[builtins.int] | None = ...,
        assigned_group_ids: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["favourite", b"favourite", "icon", b"icon", "id", b"id", "inserttime", b"inserttime", "name", b"name", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["assigned_group_ids", b"assigned_group_ids", "assigned_place_ids", b"assigned_place_ids", "favourite", b"favourite", "icon", b"icon", "id", b"id", "inserttime", b"inserttime", "name", b"name", "type", b"type"]) -> None: ...

global___Device = Device

@typing.final
class DevicesListResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEVICES_FIELD_NUMBER: builtins.int
    @property
    def devices(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Device]: ...
    def __init__(
        self,
        *,
        devices: collections.abc.Iterable[global___Device] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["devices", b"devices"]) -> None: ...

global___DevicesListResponse = DevicesListResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: login_response.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14login_response.proto\"\x85\x01\n\rLoginResponse\x12\x14\n\x0clogin_status\x18\x01 \x01(\x05\x12\x13\n\x0bprivate_key\x18\x02 \x01(\x0c\x12\x12\n\npublic_key\x18\x03 \x01(\x0c\x12\r\n\x05\x61\x64min\x18\x04 \x01(\x08\x12\x15\n\rserial_number\x18\x05 \x01(\t\x12\x0f\n\x07user_id\x18\x06 \x01(\x03')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'login_response_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_LOGINRESPONSE']._serialized_start=25
  _globals['_LOGINRESPONSE']._serialized_end=158
# @@protoc_insertion_point(module_scope)



from hashlib import sha256
from struct import pack

from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes


def create_iv(value: int) -> bytearray:
    return bytearray(b"\x00" * 12) + pack(">I", value)


def create_key(value: str) -> bytes:
    return sha256(value.encode("utf-8")).digest()


def decrypt_body(key: bytes, iv: bytes, value: bytes) -> bytes:
    decryptor = Cipher(algorithms.AES(key), modes.CFB(iv)).decryptor()
    return decryptor.update(value) + decryptor.finalize()


def encrypt_body(key: bytes, iv: bytes, value: bytes) -> bytes:
    encryptor = Cipher(algorithms.AES(key), modes.CFB(iv)).encryptor()
    return encryptor.update(value) + encryptor.finalize()



from typing import Union

from mobilus_client.proto import (
    CallEventsRequest,
    CurrentStateRequest,
    CurrentStateResponse,
    DevicesListRequest,
    DevicesListResponse,
    LoginRequest,
    LoginResponse,
)

MessageRequest = Union[
    CallEventsRequest,
    CurrentStateRequest,
    DevicesListRequest,
    LoginRequest,
]
MessageResponse = Union[
    CallEventsRequest,
    CurrentStateResponse,
    DevicesListResponse,
    LoginResponse,
]



import logging
import secrets
import socket

from mobilus_client.client import Client
from mobilus_client.config import Config
from mobilus_client.messages.serializer import MessageSerializer
from mobilus_client.registries.key import KeyRegistry
from mobilus_client.registries.message import MessageRegistry

logger = logging.getLogger(__name__)


class App:
    def __init__(self, config: Config) -> None:
        self.config = config

    def call(self, commands: list[tuple[str, dict[str, str]]]) -> str:
        if not commands:
            return self._empty_response()

        # Initialize client and registries
        key_registry = KeyRegistry(self.config.user_key)
        message_registry = MessageRegistry()

        client = Client(
            client_id=secrets.token_hex(6).upper(),
            config=self.config,
            key_registry=key_registry,
            message_registry=message_registry,
        )

        try:
            # Connect to the MQTT broker and authenticate
            if not client.connect_and_authenticate():
                return self._empty_response()

            # Execute the provided commands
            for command, params in commands:
                client.send_request(command, **params)

            # Wait for the completion event to be triggered
            client.completed_event.wait(timeout=self.config.timeout_period)
        except socket.gaierror:
            logger.error("Failed to connect to the gateway host")
        except TimeoutError:
            logger.error("Timeout occurred")
        finally:
            client.terminate()

        # Return serialized responses from the message registry
        return MessageSerializer.serialize_list_to_json(
            message_registry.get_responses(),
        )

    def _empty_response(self) -> str:
        return MessageSerializer.serialize_list_to_json([])



from __future__ import annotations

import logging
import threading
from typing import TYPE_CHECKING, Any

import paho.mqtt.client as mqtt

from mobilus_client.messages.encryptor import MessageEncryptor
from mobilus_client.messages.factory import MessageFactory
from mobilus_client.messages.status import MessageStatus
from mobilus_client.messages.validator import MessageValidator
from mobilus_client.proto import LoginRequest, LoginResponse

if TYPE_CHECKING:
    from mobilus_client.config import Config
    from mobilus_client.registries.key import KeyRegistry
    from mobilus_client.registries.message import MessageRegistry

logger = logging.getLogger(__name__)


class Client:
    def __init__(
            self, client_id: str, config: Config, key_registry: KeyRegistry, message_registry: MessageRegistry) -> None:
        self.config = config
        self.client_id = client_id
        self.shared_topic = "clients"
        self.command_topic = "module"
        self.key_registry = key_registry
        self.message_registry = message_registry
        self.authenticated_event = threading.Event()
        self.completed_event = threading.Event()

        self.mqtt_client = mqtt.Client(client_id=self.client_id, transport=config.gateway_protocol)
        self._configure_client()

    def connect_and_authenticate(self) -> bool:
        self.mqtt_client.connect(self.config.gateway_host, self.config.gateway_port)
        self.mqtt_client.loop_start()

        # Wait for the client to authenticate
        self.authenticated_event.wait(timeout=self.config.auth_timeout_period)

        if not self.authenticated_event.is_set():
            logger.error("Failed to authenticate with the gateway host")
            return False

        return True

    def send_request(self, command: str, **params: str | bytes | int | None) -> None:
        if not self.mqtt_client.is_connected():
            logger.error("Sending request - %s failed. Client is not connected.", command)
            return

        message = MessageFactory.create_message(command, **params)
        status = MessageValidator.validate(message)

        if status != MessageStatus.SUCCESS or message is None:
            logger.error("Command - %s returned an error - %s", command, status.name)
            self.terminate()
            return

        if not isinstance(message, LoginRequest):
            self.message_registry.register_request(message)

        encrypted_message = MessageEncryptor.encrypt(
            message,
            self.client_id,
            self.key_registry,
        )

        self.mqtt_client.publish(self.command_topic, encrypted_message)

    def terminate(self) -> None:
        self.mqtt_client.disconnect()
        self.mqtt_client.loop_stop()

    def on_disconnect_callback(self, _client: mqtt.Client, _userdata: None, reason_code: int) -> None:
        logger.info("Disconnected with result code - %s", reason_code)

    def on_connect_callback(
            self, _client: mqtt.Client, _userdata: None, _flags: dict[str, Any], _reason_code: int) -> None:
        self.mqtt_client.subscribe([
            (self.client_id, 0),
            (self.shared_topic, 0),
        ])

    def on_subscribe_callback(self, _client: mqtt.Client, _userdata: None, _mid: int, _granted_qos: tuple[int]) -> None:
        self.send_request(
            "login",
            login=self.config.user_login,
            password=self.config.user_key,
        )

    def on_message_callback(self, _client: mqtt.Client, _userdata: None, mqtt_message: mqtt.MQTTMessage) -> None:
        logger.info("Received message on topic - %s", mqtt_message.topic)

        # Ignore messages from the shared topic sent by other clients until the client is authenticated.
        if mqtt_message.topic == self.shared_topic and not self.authenticated_event.is_set():
            logger.info("Client is not authenticated yet. Ignoring message.")
            return

        message = MessageEncryptor.decrypt(mqtt_message.payload, self.key_registry)
        logger.info("Decrypted message - %s", type(message).__name__)

        status = MessageValidator.validate(message)

        if status != MessageStatus.SUCCESS or message is None:
            logger.error("Message - %s returned an error - %s", type(message).__name__, status.name)
            self.terminate()
            return

        logger.info("Message - %s validated successfully", type(message).__name__)

        if isinstance(message, LoginResponse):
            self.key_registry.register_keys(message)
            self.authenticated_event.set()
        else:
            self.message_registry.register_response(message)

            if self.message_registry.all_responses_received():
                self.completed_event.set()

    def _configure_client(self) -> None:
        self.mqtt_client.enable_logger(logger)
        self.mqtt_client.on_connect = self.on_connect_callback
        self.mqtt_client.on_disconnect = self.on_disconnect_callback
        self.mqtt_client.on_subscribe = self.on_subscribe_callback
        self.mqtt_client.on_message = self.on_message_callback



import json

from google.protobuf.json_format import MessageToJson

from mobilus_client.utils.types import MessageResponse


class MessageSerializer:
    @staticmethod
    def serialize_to_json(message: MessageResponse) -> str:
        return MessageToJson(message)

    @staticmethod
    def serialize_list_to_json(messages: list[MessageResponse]) -> str:
        return json.dumps(
            [
                json.loads(
                    MessageSerializer.serialize_to_json(message),
                )
                for message in messages
            ],
        )



from __future__ import annotations

import struct
import time
from typing import TYPE_CHECKING, ClassVar, cast

from google.protobuf.message import DecodeError

from mobilus_client.proto import (
    CallEventsRequest,
    CurrentStateRequest,
    CurrentStateResponse,
    DevicesListRequest,
    DevicesListResponse,
    LoginRequest,
    LoginResponse,
)
from mobilus_client.utils.encryption import create_iv, decrypt_body, encrypt_body
from mobilus_client.utils.types import MessageRequest, MessageResponse

if TYPE_CHECKING:
    from mobilus_client.registries.key import KeyRegistry


class MessageEncryptor:
    CATEGORY_MAP: ClassVar[dict[int, type[MessageRequest | MessageResponse]]] = {
        1: LoginRequest,
        2: LoginResponse,
        3: DevicesListRequest,
        4: DevicesListResponse,
        13: CallEventsRequest,
        26: CurrentStateRequest,
        27: CurrentStateResponse,
    }
    CLASS_TO_CATEGORY_MAP: ClassVar[dict[type[MessageRequest | MessageResponse], int]] = {
        v: k for k, v in CATEGORY_MAP.items()
    }

    @staticmethod
    def encrypt(message: MessageRequest, client_id: str, key_registry: KeyRegistry) -> bytes:
        category = MessageEncryptor.CLASS_TO_CATEGORY_MAP.get(type(message))
        body = message.SerializeToString()
        timestamp = int(time.time())
        length = 13

        private_key = key_registry.get_encryption_key(type(message))

        if private_key is None:
            encrypted_body = body
        else:
            iv = create_iv(timestamp)
            encrypted_body = encrypt_body(private_key, iv, body)

        return (
            struct.pack(">IBI", length, category, timestamp) +
            bytes.fromhex(client_id) +
            struct.pack(">2B", 4, 0) +
            encrypted_body
        )

    @staticmethod
    def decrypt(encrypted_message: bytes, key_registry: KeyRegistry) -> MessageResponse | None:
        # Define header format and calculate size
        header_format = ">IBI6sBB"
        header_size = struct.calcsize(header_format)

        # Do early return if message is too short
        if len(encrypted_message) < header_size:
            return None

        # Unpack header
        length, category, timestamp, user_id, platform, response_code = struct.unpack(
            header_format, encrypted_message[:header_size])

        # Extract encrypted body
        encrypted_body = encrypted_message[header_size:]

        # Choose proper klass
        message_klass = MessageEncryptor.CATEGORY_MAP.get(category)

        if message_klass is None:
            return None

        message_klass = cast(type[MessageResponse], message_klass)

        # Choose proper decryption key
        key = key_registry.get_decryption_key(message_klass)

        # Decrypt body
        iv = create_iv(timestamp)
        body = decrypt_body(key, iv, encrypted_body)

        try:
            message = message_klass()
            message.ParseFromString(body)
        except DecodeError:
            return None

        return message



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class LoginRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOGIN_FIELD_NUMBER: builtins.int
    PASSWORD_FIELD_NUMBER: builtins.int
    login: builtins.str
    password: builtins.bytes
    def __init__(
        self,
        *,
        login: builtins.str | None = ...,
        password: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["login", b"login", "password", b"password"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["login", b"login", "password", b"password"]) -> None: ...

global___LoginRequest = LoginRequest



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: call_events_request.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x63\x61ll_events_request.proto\"\x83\x01\n\tCallEvent\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x11\n\tdevice_id\x18\x02 \x01(\x03\x12\x14\n\x0c\x65vent_number\x18\x03 \x01(\x05\x12\r\n\x05value\x18\x04 \x01(\t\x12\x10\n\x08platform\x18\x05 \x01(\x05\x12\x0c\n\x04user\x18\x06 \x01(\x03\x12\x12\n\ninserttime\x18\x07 \x01(\x03\"/\n\x11\x43\x61llEventsRequest\x12\x1a\n\x06\x65vents\x18\x01 \x03(\x0b\x32\n.CallEvent')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'call_events_request_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_CALLEVENT']._serialized_start=30
  _globals['_CALLEVENT']._serialized_end=161
  _globals['_CALLEVENTSREQUEST']._serialized_start=163
  _globals['_CALLEVENTSREQUEST']._serialized_end=210
# @@protoc_insertion_point(module_scope)



import argparse
import json
import logging
import sys

from mobilus_client.app import App
from mobilus_client.config import Config

logger = logging.getLogger(__name__)


def main() -> None:
    parser = argparse.ArgumentParser(
        description="Run Mobilus Client with specific commands and parameters.",
    )
    parser.add_argument("--host", required=True, help="Cosmo GTW address")
    parser.add_argument("--login", required=True, help="User login")
    parser.add_argument("--password", required=True, help="User password")
    parser.add_argument("--verbose", action="store_true", help="Enable verbose logging")
    parser.add_argument("commands", nargs="+", help="Commands to run")

    # Parse arguments
    args = parser.parse_args()

    # Create a config object
    config = Config(
        gateway_host=args.host,
        user_login=args.login,
        user_password=args.password,
    )

    # Set up logging
    logging.basicConfig(
        stream=sys.stdout,
        level=logging.DEBUG if args.verbose else logging.WARNING,
        format=json.dumps(
            {
                "name": "[%(name)s]",
                "levelname": "%(levelname)s",
                "message": "%(message)s",
            },
        ),
    )

    commands = []
    for command_str in args.commands:
        command, params = _parse_command(command_str)
        commands.append((command, params))

    app = App(config)
    responses = app.call(commands)
    sys.stdout.write(responses)


def _parse_command(command_str: str) -> tuple[str, dict[str, str]]:
    """" Parse a command string into a command and a dictionary of parameters. "
    " Example: 'command:param1=value1,param2=value2' "
    " Returns: ('command', {'param1': 'value1', 'param2': 'value2'}) "
    """
    command, *params = command_str.split(":", 1)
    params_dict = dict(param.split("=") for param in params[0].split(",")) if params else {}
    return command, params_dict


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: current_state_response.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x63urrent_state_response.proto\"\x8b\x01\n\x11\x43urrentStateEvent\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x11\n\tdevice_id\x18\x02 \x01(\x03\x12\x14\n\x0c\x65vent_number\x18\x03 \x01(\x05\x12\r\n\x05value\x18\x04 \x01(\t\x12\x10\n\x08platform\x18\x05 \x01(\x05\x12\x0c\n\x04user\x18\x06 \x01(\x03\x12\x12\n\ninserttime\x18\x07 \x01(\x03\":\n\x14\x43urrentStateResponse\x12\"\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x12.CurrentStateEvent')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'current_state_response_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_CURRENTSTATEEVENT']._serialized_start=33
  _globals['_CURRENTSTATEEVENT']._serialized_end=172
  _globals['_CURRENTSTATERESPONSE']._serialized_start=174
  _globals['_CURRENTSTATERESPONSE']._serialized_end=232
# @@protoc_insertion_point(module_scope)



from mobilus_client.proto.call_events_request_pb2 import CallEventsRequest, CallEvent
from mobilus_client.proto.current_state_request_pb2 import CurrentStateRequest
from mobilus_client.proto.current_state_response_pb2 import CurrentStateResponse, CurrentStateEvent
from mobilus_client.proto.devices_list_request_pb2 import DevicesListRequest
from mobilus_client.proto.devices_list_response_pb2 import DevicesListResponse, Device
from mobilus_client.proto.login_request_pb2 import LoginRequest
from mobilus_client.proto.login_response_pb2 import LoginResponse

__all__ = [
    "CallEvent",
    "CallEventsRequest",
    "CurrentStateRequest",
    "CurrentStateResponse",
    "CurrentStateEvent",
    "Device",
    "DevicesListRequest",
    "DevicesListResponse",
    "LoginRequest",
    "LoginResponse"
]



from collections import Counter
from typing import ClassVar

from mobilus_client.proto import (
    CallEventsRequest,
    CurrentStateRequest,
    CurrentStateResponse,
    DevicesListRequest,
    DevicesListResponse,
    LoginRequest,
    LoginResponse,
)
from mobilus_client.utils.types import MessageRequest, MessageResponse


class MessageRegistry:
    MESSAGE_MAP: ClassVar[dict[type[MessageRequest], type[MessageResponse]]] = {
        CallEventsRequest: CallEventsRequest,
        CurrentStateRequest: CurrentStateResponse,
        DevicesListRequest: DevicesListResponse,
        LoginRequest: LoginResponse,
    }

    def __init__(self) -> None:
        self._requests: list[MessageRequest] = []
        self._responses: list[MessageResponse] = []

    def register_request(self, message_request: MessageRequest) -> None:
        self._requests.append(message_request)

    def register_response(self, message_response: MessageResponse) -> None:
        self._responses.append(message_response)

    def get_requests(self) -> list[MessageRequest]:
        return self._requests

    def get_responses(self) -> list[MessageResponse]:
        return self._responses

    def all_responses_received(self) -> bool:
        expected_responses = Counter(self.MESSAGE_MAP[type(request)] for request in self.get_requests())
        actual_responses = Counter(type(response) for response in self.get_responses())

        return expected_responses == actual_responses



from dataclasses import dataclass

from mobilus_client.utils.encryption import create_key


@dataclass
class Config:
    gateway_host: str
    user_login: str
    user_password: str

    auth_timeout_period: float = 30
    gateway_port: int = 8884
    gateway_protocol: str = "websockets"
    timeout_period: float = 30
    user_key: bytes = b""

    def __post_init__(self) -> None:
        self.user_key = create_key(self.user_password)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class LoginResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOGIN_STATUS_FIELD_NUMBER: builtins.int
    PRIVATE_KEY_FIELD_NUMBER: builtins.int
    PUBLIC_KEY_FIELD_NUMBER: builtins.int
    ADMIN_FIELD_NUMBER: builtins.int
    SERIAL_NUMBER_FIELD_NUMBER: builtins.int
    USER_ID_FIELD_NUMBER: builtins.int
    login_status: builtins.int
    private_key: builtins.bytes
    public_key: builtins.bytes
    admin: builtins.bool
    serial_number: builtins.str
    user_id: builtins.int
    def __init__(
        self,
        *,
        login_status: builtins.int | None = ...,
        private_key: builtins.bytes | None = ...,
        public_key: builtins.bytes | None = ...,
        admin: builtins.bool | None = ...,
        serial_number: builtins.str | None = ...,
        user_id: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["admin", b"admin", "login_status", b"login_status", "private_key", b"private_key", "public_key", b"public_key", "serial_number", b"serial_number", "user_id", b"user_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["admin", b"admin", "login_status", b"login_status", "private_key", b"private_key", "public_key", b"public_key", "serial_number", b"serial_number", "user_id", b"user_id"]) -> None: ...

global___LoginResponse = LoginResponse



from __future__ import annotations

from typing import TYPE_CHECKING

from mobilus_client.messages.status import MessageStatus
from mobilus_client.proto import LoginResponse

if TYPE_CHECKING:
    from mobilus_client.utils.types import MessageRequest, MessageResponse


class MessageValidator:
    @staticmethod
    def validate(message: MessageRequest | MessageResponse | None) -> MessageStatus:
        if message is None:
            return MessageStatus.UNKNOWN_MESSAGE

        if isinstance(message, LoginResponse) and message.login_status == 1:
            return MessageStatus.AUTHENTICATION_ERROR

        return MessageStatus.SUCCESS



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: devices_list_response.proto
# Protobuf Python Version: 4.25.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x64\x65vices_list_response.proto\"\x9d\x01\n\x06\x44\x65vice\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\x05\x12\x0c\n\x04icon\x18\x04 \x01(\x05\x12\x12\n\ninserttime\x18\x05 \x01(\x03\x12\x11\n\tfavourite\x18\x06 \x01(\x08\x12\x1a\n\x12\x61ssigned_place_ids\x18\x07 \x03(\x03\x12\x1a\n\x12\x61ssigned_group_ids\x18\x08 \x03(\x03\"/\n\x13\x44\x65vicesListResponse\x12\x18\n\x07\x64\x65vices\x18\x01 \x03(\x0b\x32\x07.Device')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'devices_list_response_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_DEVICE']._serialized_start=32
  _globals['_DEVICE']._serialized_end=189
  _globals['_DEVICESLISTRESPONSE']._serialized_start=191
  _globals['_DEVICESLISTRESPONSE']._serialized_end=238
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, ClassVar

from mobilus_client.proto import (
    CallEvent,
    CallEventsRequest,
    CurrentStateRequest,
    DevicesListRequest,
    LoginRequest,
)

if TYPE_CHECKING:
    from mobilus_client.utils.types import MessageRequest


class CallEventsMessageBuilder:
    def build(self, device_id: str = "", value: str = "", event_number: str = "6") -> CallEventsRequest | None:
        if not device_id or not value:
            return None

        event = CallEvent()
        event.event_number = int(event_number)
        event.device_id = int(device_id)
        event.value = value

        request = CallEventsRequest()
        request.events.append(event)
        return request


class CurrentStateMessageBuilder:
    def build(self) -> CurrentStateRequest:
        return CurrentStateRequest()


class DevicesListMessageBuilder:
    def build(self) -> DevicesListRequest:
        return DevicesListRequest()


class LoginMessageBuilder:
    def build(self, login: str = "", password: bytes = b"") -> LoginRequest | None:
        if not login or not password:
            return None

        request = LoginRequest()
        request.login = login
        request.password = password
        return request


class MessageFactory:
    _builders: ClassVar[
      dict[str, LoginMessageBuilder | DevicesListMessageBuilder | CurrentStateMessageBuilder | CallEventsMessageBuilder]
     ] = {
        "call_events": CallEventsMessageBuilder(),
        "current_state": CurrentStateMessageBuilder(),
        "devices_list": DevicesListMessageBuilder(),
        "login": LoginMessageBuilder(),
    }

    @staticmethod
    def create_message(message_name: str, **kwargs: Any) -> MessageRequest | None: # noqa: ANN401
        builder = MessageFactory._builders.get(message_name)
        return builder.build(**kwargs) if builder else None



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class CurrentStateEvent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    DEVICE_ID_FIELD_NUMBER: builtins.int
    EVENT_NUMBER_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    PLATFORM_FIELD_NUMBER: builtins.int
    USER_FIELD_NUMBER: builtins.int
    INSERTTIME_FIELD_NUMBER: builtins.int
    id: builtins.int
    device_id: builtins.int
    event_number: builtins.int
    value: builtins.str
    platform: builtins.int
    user: builtins.int
    inserttime: builtins.int
    def __init__(
        self,
        *,
        id: builtins.int | None = ...,
        device_id: builtins.int | None = ...,
        event_number: builtins.int | None = ...,
        value: builtins.str | None = ...,
        platform: builtins.int | None = ...,
        user: builtins.int | None = ...,
        inserttime: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["device_id", b"device_id", "event_number", b"event_number", "id", b"id", "inserttime", b"inserttime", "platform", b"platform", "user", b"user", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["device_id", b"device_id", "event_number", b"event_number", "id", b"id", "inserttime", b"inserttime", "platform", b"platform", "user", b"user", "value", b"value"]) -> None: ...

global___CurrentStateEvent = CurrentStateEvent

@typing.final
class CurrentStateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EVENTS_FIELD_NUMBER: builtins.int
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CurrentStateEvent]: ...
    def __init__(
        self,
        *,
        events: collections.abc.Iterable[global___CurrentStateEvent] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["events", b"events"]) -> None: ...

global___CurrentStateResponse = CurrentStateResponse



from __future__ import annotations

from typing import TYPE_CHECKING

from mobilus_client.proto import CallEventsRequest, LoginResponse

if TYPE_CHECKING:
    from mobilus_client.utils.types import MessageRequest, MessageResponse


class KeyRegistry:
    def __init__(self, user_key: bytes) -> None:
        self._registry = {
            "user_key": user_key,
        }

    def get_keys(self) -> dict[str, bytes]:
        return self._registry

    def register_keys(self, message: LoginResponse) -> None:
        self._registry["private_key"] = message.private_key
        self._registry["public_key"] = message.public_key

    def get_decryption_key(self, message_klass: type[MessageResponse]) -> bytes:
        if message_klass == LoginResponse:
            return self._registry["user_key"]

        if message_klass == CallEventsRequest:
            return self._registry["public_key"]

        return self._registry["private_key"]

    def get_encryption_key(self, message_klass: type[MessageRequest]) -> bytes | None:
        if message_klass == CallEventsRequest:
            return self._registry.get("private_key")

        return None
